try:
    from django.contrib.staticfiles.templatetags.staticfiles import (
        static as django_static,
    )
except ImportError:
    from django.templatetags.static import static as django_static


def static(path):
    """
    Use the Django builtin static file resolver to return an absolute path
    usable as CSS url() argument. Sass equivalent of the 'static' template
    tag.
    """
    return '"{}"'.format(django_static(path))
