from ocyan.core.fender import config

__all__ = ["is_configured_theme"]


def is_configured_theme(theme_name):
    """
    Is 'theme_name' the theme configured in the ocyan.json file.
    """
    assert theme_name
    return config.get("theme", "name", "") == theme_name
