from pathlib import Path

from setuptools import find_packages, setup

PACKAGE_CLASSIFIERS = [
    "Framework :: Ocyan",
]

install_requires: list = [
    "Django>=5,<6",
    "straight.plugin>=1.5.1.post1",
    "configtype",
]

extras_require: dict = {
    "test": [
        "wheel",
        "empty_testproject",
        "pylint-django",
        "isort",
        "ruff",
        "vdt.versionplugin.wheel",
        "ocyan.plugin.testing",
        "coverage",
    ]
}

setup(
    name="ocyan.core",
    version="1.2.8",
    url="https://git.mandelblog.com/salt/ocyan.core",
    author="Motolani Olaiya",
    author_email="motolaniolaiya@gmail.com",
    description="A framework for ocyan plugins, all plugins depend on this.",
    long_description=Path("README.md").read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.12,<3.13",
    install_requires=install_requires,
    classifiers=PACKAGE_CLASSIFIERS,
    extras_require=extras_require,
)
